package bricksplanner

import (
	smartvolapi "github.com/gluster/glusterd2/plugins/smartvol/api"
)

type replicaSubvolPlanner struct {
	subvolSize       uint64
	replicaCount     int
	arbiterCount     int
	brickSize        uint64
	arbiterBrickSize uint64
}

func (s *replicaSubvolPlanner) Init(req *smartvolapi.Volume, subvolSize uint64) {
	s.subvolSize = subvolSize
	s.replicaCount = req.ReplicaCount
	s.arbiterCount = req.ArbiterCount
	s.brickSize = s.subvolSize
	// TODO: Calculate Arbiter brick size
	s.arbiterBrickSize = s.subvolSize
}

func (s *replicaSubvolPlanner) BricksCount() int {
	return s.replicaCount
}

func (s *replicaSubvolPlanner) BrickSize(idx int) uint64 {
	if idx == (s.replicaCount-1) && s.arbiterCount > 0 {
		return s.arbiterBrickSize
	}

	return s.brickSize
}

func (s *replicaSubvolPlanner) BrickType(idx int) string {
	if idx == (s.replicaCount-1) && s.arbiterCount > 0 {
		return "arbiter"
	}

	return "brick"
}

func init() {
	subvolPlanners["replicate"] = &replicaSubvolPlanner{}
}
